<?php

if(!isset($argv[1]) || !isset($argv[2]) || 
!preg_match("/\.svg$/",$argv[1]) || !preg_match("/\.svg$/",$argv[2]) ||
!file_exists ($argv[1]) || file_exists($argv[2]) ) {
	print "Usage: php tspchomper.php <inputfile> <outputfile>
The two files must be SVGs, The first filename must exist, and the second one must not exist.

The first filename needs to be a simple SVG that consists entirely 
of separate straight line path objects, no other types of objects are supported.

No action has been taken at this time, your arguments were incorrect.";
exit;
}
/*
    <path
       style="fill:none;stroke:#000000;stroke-width:0.26458338px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"
       d="m 153.08037,117.46129 h 25.4"
       id="path812"
       inkscape:connector-curvature="0" />
    <path
       style="fill:none;stroke:#0000ff;stroke-width:0.26458338px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"
       d="m 149.76975,128.70276 17.96052,17.96052"
       id="path812-5"
       inkscape:connector-curvature="0" />
    <path
       style="fill:none;stroke:#006400;stroke-width:0.26458338px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"
       d="m 157.04911,151.06337 v 25.40002"
       id="path812-5-5"
       inkscape:connector-curvature="0" />

  */
  
  // ok lets get our paths.
  
  
  if (file_exists($argv[1])) {
    $xml = simplexml_load_file($argv[1]);

$howmany = count($xml->g->children());
$tcount=0;
foreach ($xml->g->children() as $tkey => $child) {
	
	// now we have them.

/*
	SimpleXMLElement Object
(
    [@attributes] => Array
        (
            [style] => fill:none;stroke:#000000;stroke-width:0.25
            [d] => m 435.575,152.572 -3.148,-5.01
            [id] => path24822
        )

)
*/
// write to the actual xml array...
//$xml->g->path[$tcount]['id']) = blahblah;

$lengths[(string)$child['id']] = getlength($child['d']);

$tcount++;

}


// now we have an array of lengths.  Let's do stuff to that.

// let's figure out what the average is and put some sensibility around the lengths.  
// It might be more sensible to modify this to be a percentile calculator to get a better spread of colors, 
// but this is an artistic choice, and is up to you.
$longest = max($lengths);
$shortest = min($lengths);
$averagelength = array_sum($lengths)/count($lengths);

// the range I care about.  You can mess with this to tune your results.
$realrange = ($averagelength-$shortest)*2;

// this equates to how many colors you'll end up with, adjust this as you see fit.
$steps = 15;

// This is how much to up the color value.  The colors range from pure blue to pure blue green.  
// A proper HSV to hex code function would be grand here, but no time for that.
$stepfactor = floor(250/$steps);
$stepsize = $realrange/$steps;
// This is how much to step up the line width.  
// As for the glowforge, this is irrelevant but it does make for a more obvious visual presentation.
$widthincrement = 0.25;

// cycle through the children again, we're going to modify them now.
$tcount = 0;
foreach ($xml->g->children() as $tkey => $child) {

if(strlen((string) $child['id']) == 0) {
	// null element, skip it.
	continue;
}
// grab the length form our array.
$tlength = $lengths[(string) $child['id']];

// use that length to figure out how much to modify the color and line weight values:
$numsteps = min(floor(abs($shortest-$tlength)/$stepsize),$steps); // capped at $steps

//print "Stepsize: $stepsize shortest: $shortest length: $length numsteps $numsteps \n";

// make a green value to pass into the hex code:
$greenchannel = dechex($stepfactor*$numsteps);

// make your width to pass into the style value
$twidth = ($steps * $widthincrement) - ($widthincrement*$numsteps) + $widthincrement;
if(strlen($greenchannel) == 1) {
	$greenchannel = "0".$greenchannel;
}
// here's my hex code.
$color = "00".$greenchannel."ff";

// I just hardcoded a lot of this, because it was quick and dirty.
$styleline = 'fill:none;stroke:#'.$color.';stroke-width:'.$twidth.';stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1';

// now set that style line that incorprates the color and width.
$xml->g->path[$tcount]['style'] = $styleline;

  
  $tcount++;
  }
  
  // output the new SVG to your filename in the arguments.
  $xml->asXML($argv[2]);
  print "Wrote to {$argv[2]}\n";

  

  }
  else {
	  print "file not found.\n";
  }
  

    function getlength($dimstr) {
	  	$dimarr = explode(" ",$dimstr);
//print "$dimstr\n";
//print_r($dimarr);
	$endpoint = trim(array_pop($dimarr));
	$startpoint = trim($dimarr[1]);		
	$movetype = trim($dimarr[0]);
	if(strstr($endpoint,',')) {
		// it's coordinates.
		list($startx, $starty) = explode(',',$startpoint);
		list($endx, $endy) = explode(',',$endpoint);

		$startx = $startx + 0;
		$starty = $starty + 0;
		$endx = $endx + 0;
		$endy = $endy + 0;
		if(strcmp($movetype,"m") == 0) {
			list($xdiff,$ydiff) = explode(',',$endpoint);
		}
		else {
			//print "This is an absolute move, calculating xdiff and ydiff\n";
			//print " abs($startx - $endx);\nabs($starty - $endy);\n";
			$xdiff= abs($startx - $endx);
			$ydiff= abs($starty - $endy);
		}
		//print "sqrt(($xdiff*$xdiff)+($ydiff*$ydiff))\n";
		$length = sqrt(($xdiff*$xdiff)+($ydiff*$ydiff));
	}
	else {
//		print "endpoint $endpoint\n";
//		print $dimstr;
		$endpoint = $endpoint+0;
		$length = abs($endpoint);
	}
	$length = round($length,3);
	return $length;
  }
  
  
?>